local ADDON_NAME = ...

-----------------------------
-- Frame

local StacksFrame = CreateFrame("Frame", "MalygosStacksFrame", UIParent)

local StacksFrame_OnUpdate
do
	local debuffName = GetSpellInfo(56092) -- Engulf in Flames

	-- returns a color in a gradiant from green (1) to red (0)
	local function grad(ratio)
		ratio = math.min(1, ratio)
		-- (1 + ((0) - 1) * ratio) * 255, (0 + ((1) - 0) * ratio) * 255), 0
		return string.format("%02x%02x00", (1 - ratio) * 255, ratio * 255)
	end

	function StacksFrame_OnUpdate(self, elapsed)
		local power = UnitPower("vehicle")
		if power >= 50 then
			self.background:SetVertexColor(1, 1, 1, 1)
		else
			-- ramp up alpha wrt energy from 30 to 50, providing a visual jump (.68 -> 1) at 50
			self.background:SetVertexColor(1, 1, 1, math.max(0.3, (power - 15) * 2 / 100))
		end

		local _, _, _, count, _, _, expires = UnitDebuff("target", debuffName, nil, "PLAYER")
		if expires then
			-- color the text based on the remaining time
			local remaining = expires - GetTime()
			self.time:SetFormattedText("|cff%s%0.1f s|r", grad(remaining / 15), remaining)
			self.time:Show()
		else
			self.time:Hide()
		end

		self.count:SetFormattedText("|cffffffff%d|r", count or 0)
	end
end

local function SetupFrame()
	local frame = StacksFrame
	frame:SetWidth(64)
	frame:SetHeight(64)
	frame:ClearAllPoints()
	frame:SetPoint(MalygosStacksDB.point or "CENTER", MalygosStacksDB.x or 0, MalygosStacksDB.y or 0)

	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetClampedToScreen(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", frame.StartMoving)
	frame:SetScript("OnDragStop", function(self)
		self:StopMovingOrSizing()
		local point, _, _, x, y = self:GetPoint(1)
		MalygosStacksDB.point = point
		MalygosStacksDB.x = x
		MalygosStacksDB.y = y
	end)

	local texture = frame:CreateTexture("MalygosStacksFrameBackground", "BACKGROUND")
	texture:SetAllPoints(frame)
	texture:SetTexture("Interface\\Icons\\Spell_Fire_Burnout")
	frame.background = texture

	local text1 = frame:CreateFontString("MalygosStacksFrameTimeText", "OVERLAY")
	text1:ClearAllPoints()
	text1:SetPoint("TOP", frame, "BOTTOM", 0, 0)
	text1:SetJustifyH("CENTER")
	text1:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
	frame.time = text1

	local text2 = frame:CreateFontString("MalygosStacksFrameCountText", "OVERLAY")
	text2:SetWidth(64)
	text2:SetHeight(64)
	text2:ClearAllPoints()
	text2:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
	text2:SetJustifyH("CENTER")
	text2:SetJustifyV("MIDDLE")
	text2:SetFont("Fonts\\FRIZQT__.TTF", 36, "OUTLINE")
	text2:SetText("|cFFFFFFFF0|r")
	frame.count = text2

	local comboPoints = {}
	for i = 1, 5 do
		local combo = frame:CreateTexture("MalygosStacksFrameCombo" .. i, "OVERLAY")
		combo:SetTexture("Interface\\AddOns\\MalygosStacks\\combo") -- texture from PitBull4_ComboPoints
		combo:SetWidth(12)
		combo:SetHeight(12)
		if i == 1 then
			combo:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -2, 0)
		else
			combo:SetPoint("RIGHT", comboPoints[i-1], "LEFT")
		end
		combo:Hide()
		comboPoints[i] = combo
	end
	frame.comboPoints = comboPoints

	frame:SetScript("OnUpdate", StacksFrame_OnUpdate)
end


-----------------------------
-- Events

StacksFrame:SetScript("OnEvent", function(self, event, ...)
	self[event](self, ...)
end)

function StacksFrame:UNIT_ENTERED_VEHICLE(unit)
	if unit == "player" then
		local _, _, _, _, _, mobId = strsplit("-", UnitGUID("vehicle"))
		if mobId == 30161 or mobId == 32535 then
			if SetupFrame then
				SetupFrame()
				SetupFrame = nil
			end
			self:Show()
			self:RegisterEvent("PLAYER_TARGET_CHANGED")
			self:RegisterUnitEvent("UNIT_POWER_FREQUENT", "player", "vehicle")
			self:RegisterUnitEvent("UNIT_EXITED_VEHICLE", "player")
		end
	end
end

function StacksFrame:UNIT_EXITED_VEHICLE(unit)
	if unit == "player" and self:IsShown() then
		self:UnregisterEvent("PLAYER_TARGET_CHANGED")
		self:UnregisterEvent("UNIT_POWER_FREQUENT")
		self:UnregisterEvent("UNIT_EXITED_VEHICLE")
		self:Hide()
	end
end

function StacksFrame:UNIT_POWER_FREQUENT(unit, powerType)
	if not UnitInVehicle("player") then
		self:UNIT_EXITED_VEHICLE("player")
		return
	end
	if powerType == "COMBO_POINTS" then
		local comboPoints = GetComboPoints("vehicle")
		for i = 1, 5 do
			if comboPoints >= i then
				self.comboPoints[i]:Show()
			else
				self.comboPoints[i]:Hide()
			end
		end
	end
end

function StacksFrame:PLAYER_TARGET_CHANGED()
	self:UNIT_POWER_FREQUENT("player", "COMBO_POINTS")
end


function StacksFrame:ADDON_LOADED(addon)
	if addon ~= ADDON_NAME then return end

	MalygosStacksDB = MalygosStacksDB or {}
	self:RegisterUnitEvent("UNIT_ENTERED_VEHICLE", "player")

	if IsLoggedIn() then
		self:UNIT_ENTERED_VEHICLE("player")
	end

	self:UnregisterEvent("ADDON_LOADED")
	self.ADDON_LOADED = nil
end
StacksFrame:RegisterEvent("ADDON_LOADED")


SLASH_MALYGOSSTACKS1 = "/malygosstacks"
SlashCmdList["MALYGOSSTACKS"] = function()
	StacksFrame:UNIT_ENTERED_VEHICLE("player")
end
